<?xml version="1.0" encoding="UTF-8"?>
<html xmlns="http://www.w3.org/1999/xhtml" lang="en">
  <head>
    <title>${title}</title>
    <link type="text/css" rel="stylesheet" href="../stylesheet.css"/>
  </head>
  <body>
    <h2>${title}</h2>
    <h3>By ${author}</h3>
    <h4>Published on ${date}</h4>
    ${content}
  </body>
</html>